package version

// Version is the version of the build.
// NOTE: remember to bump the version at the top
// of the top-level README.md file when this is
// bumped.
const Version = "2.0.0-rc7"

// APIVersion is the version for the remote
// client API.  It is used to determine compatibility
// between a remote podman client and its backend
const APIVersion = 1
